import json
import csv
import os


class EVALtask2(object):

    def __init__(self, pred_filename=None, list_dirname=None,
                 subset='val'):
        if not pred_filename:
            raise IOError('Please input a valid prediction file.')
        if not list_dirname:
            raise IOError('Please input a valid directory.')
        if not ((subset == "val") or (subset == "test")):
            raise IOError('Please input subset as "val" or "test"')

        self.subset = subset
        self.pred_filename = pred_filename
        self.list_dirname = list_dirname

        self.topk = [1,5]
        self.ap = None
        # TODO verbose
        self.verbose = True

    def evaluate(self):

        list_path = os.path.join(self.list_dirname, (self.subset + "_list.csv"))

        # load list file
        with open(list_path, "r") as f:
            reader = csv.reader(f)
            target_list = [row for row in reader]

        with open(self.pred_filename, 'r') as fobj:
            pred_data = json.load(fobj)
        # Checking format...
        pred_fields = ['version', 'results', 'external_data']
        if not all([field in pred_data.keys() for field in pred_fields]):
            raise IOError('Please input a valid prediction file.')

        # list includes ID(without view_id), activityname
        score = [0, 0]
        idx = 0
        for one_video in target_list:
            video_id = one_video[0]
            gt_activity = one_video[1]
            one_pred = pred_data['results'][video_id]
            pred_candidate = {}
            for one in one_pred:
                pred_candidate[one['label']] = one['score']
            sorted_pred = []
            for k, _ in sorted(pred_candidate.items(), key=lambda x: -x[1]):
                sorted_pred.append(k)
            print(sorted_pred)
            for i, k in enumerate(self.topk):
                if len(sorted_pred) < k:
                    print('[Warning] Your prediction file does not have enough candidate.')
                    k = len(sorted_pred)
                if gt_activity in sorted_pred[0:k]:
                    score[i] += 1
            idx += 1

        score_1 = score[0] / idx
        score_5 = score[1] / idx
        score_all = (score_1 + score_5) / 2

        if self.verbose:
            print('<<RESULTS>> Home Action Genome Task #2')
            print('\tAccuracy: {:.3g}'.format(score_all))


